class L2GPAIDirector extends Actor
    placeable;

enum EMapType
{
    MT_Campaign,
    MT_Survival,
};

enum EHealthState
{
    HS_Healthy,
    HS_Injured,
    HS_Critical,
};

struct range
{
    var() float Min;
    var() float Max;
};

var() EMapType MapType;

var() int SurvivorHealthyThreshold;
var() int SurvivorInjuredThreshold;
var() int SurvivorCriticalThreshold;

var() float SpawnRadius;

var() range NormalSpawnDuration;
var() range HoardDuration;
var() range RestDuration;

var array<PersonPawn> Survivors;
var array<L2GPZombieSpawner> ZombieSpawners;

simulated function PostBeginPlay()
{
    FindZombieSpawners();

    log("Initialized AI Director");

    // No matter what game, can't start before
    // survivors join the server.
    SetTimer(1.0, true);
}

simulated function Timer()
{
    PopulateSurvivorList();

        if (Survivors.length > 0)
        {
            switch (MapType)
            {
                case MT_Campaign:    SetNormalState();
                                     break;

                case MT_Survival:    SetHoardState();
                                     break;
            }
        }
}

simulated function RefreshSurvivorList()
{
    ClearSurvivorList();
    PopulateSurvivorList();
}

simulated function ClearSurvivorList()
{
    while (Survivors.length > 0)
        Survivors.Remove(Survivors.length-1, 1);
}

simulated function PopulateSurvivorList()
{
    local int i;
    local PersonPawn Survivor;

    foreach AllActors(class'PersonPawn', Survivor)
        if (Survivor != None)
        {
            Survivors.Insert(Survivors.length,1);
            Survivors[Survivors.length-1] = Survivor;
        }

    for (i=0;i<Survivors.length;i++)
        log("Added "$string(Survivors[i])$" to the Survivor List");
}

simulated function SetNormalState()
{
    local int i;
    local L2GPZombieSpawner ZombieSpawner;

    RefreshSurvivorList();

    for (i=0;i<ZombieSpawners.length;i++)
        if (ZombieSpawners[i] != None)
            ZombieSpawners[i].GotoState('NormalSpawn');

    //for (i=0;i<Survivors.length;i++)
        //foreach RadiusActors(class'L2GPZombieSpawner', ZombieSpawner, SpawnRadius, Survivors[i].Location)
            //if (ZombieSpawner != None && !ZombieSpawner.IsInState('NormalSpawn'))
                //ZombieSpawner.GotoState('NormalSpawn');

    GotoState('NormalSpawn');
}

simulated function SetHoardState()
{
    local int i;
    local L2GPZombieSpawner ZombieSpawner;

    RefreshSurvivorList();

    for (i=0;i<ZombieSpawners.length;i++)
        if (ZombieSpawners[i] != None)
            ZombieSpawners[i].GotoState('HoardSpawn');

    //for (i=0;i<Survivors.length;i++)
        //foreach RadiusActors(class'L2GPZombieSpawner', ZombieSpawner, SpawnRadius, Survivors[i].Location)
            //if (ZombieSpawner != None && !ZombieSpawner.IsInState('HoardSpawn'))
                //ZombieSpawner.GotoState('HoardSpawn');

    GotoState('Hoard');
}

simulated function SetRestState()
{
    local int i;

    for (i=0;i<ZombieSpawners.length;i++)
        ZombieSpawners[i].GotoState('Rest');

    GotoState('Rest');
}

simulated function FindZombieSpawners()
{
    local L2GPZombieSpawner ZombieSpawner;

    foreach AllActors(class'L2GPZombieSpawner', ZombieSpawner)
        if (ZombieSpawner != None)
        {
            ZombieSpawners.Insert(ZombieSpawners.length,1);
            ZombieSpawners[ZombieSpawners.length-1] = ZombieSpawner;
            ZombieSpawner.Initialize(self);
            //log("Added: "$string(ZombieSpawner)$" to list.");
        }
}

simulated function float GetAverageHealth()
{
    local int i;
    local float CombinedHealth;
    local PersonPawn Survivor;

    for (i=0;i<Survivors.length;i++)
        if (Survivors[i] != None)
            CombinedHealth += Survivors[i].Health;

    return CombinedHealth / Survivors.length;
}

simulated function float GetHighestHealth()
{
    local int i;
    local float HighestHealth;

    HighestHealth = 0;

    for (i=0;i<Survivors.length;i++)
        if (Survivors[i] != None && Survivors[i].Health > HighestHealth)
            HighestHealth = Survivors[i].Health;

    return HighestHealth;
}

simulated function EHealthState GetHighestHealthState()
{
    local float HighestHealth;

    HighestHealth = GetHighestHealth();

    if (HighestHealth > SurvivorHealthyThreshold)
        return HS_Healthy;
    else if (HighestHealth < SurvivorHealthyThreshold && HighestHealth > SurvivorInjuredThreshold)
        return HS_Injured;
    else if (HighestHealth < SurvivorCriticalThreshold)
        return HS_Critical;
}

simulated function Pawn GetRandomSurvivor()
{
    return Survivors[Rand(Survivors.length)];
}

simulated function float GetRange(range Range)
{
    return Range.Min + (FRand() * (Range.Max - Range.Min));
}

simulated state NormalSpawn
{
Begin:
    Sleep(GetRange(NormalSpawnDuration));

    switch (GetHighestHealthState())
    {
        case HS_Healthy:    SetHoardState();
                            break;

        case HS_Injured:    SetHoardState();
                            break;

        case HS_Critical:   Goto 'Begin';
                            break;
    }
}

simulated state Hoard
{
Begin:
    if (MapType != MT_Survival)
    {
        Sleep(GetRange(HoardDuration));

        switch (GetHighestHealthState())
        {
            case HS_Healthy:    Goto 'Begin';
                                break;

            case HS_Injured:    SetNormalState();
                                break;

            case HS_Critical:   SetRestState();
                                break;
        }
    }
}

simulated state Rest
{
Begin:
    Sleep(GetRange(RestDuration));

    SetNormalState();
}

defaultproperties
{
     SurvivorHealthyThreshold=80
     SurvivorInjuredThreshold=40
     SurvivorCriticalThreshold=20
     SpawnRadius=2048.000000
     NormalSpawnDuration=(Min=60.000000,Max=90.000000)
     HoardDuration=(Min=30.000000,Max=50.000000)
     RestDuration=(Min=30.000000,Max=50.000000)
     bHidden=True
     Texture=Texture'Engine.S_Camera'
     bEdShouldSnap=True
}
