////////////////////////////////////////////
// Res-Studios.net
///////////////////////////////////////////
// Most bosses are heavily customized, but here we define some things they have
// in common, such as scaled health.
class L2GPBossZombie extends MpPawn;

struct AttackInfo
{
    var name AttackAnim;
    var float AttackDelay;
    var float AttackDuration;
    var float AttackDamage;
    var class<DamageType> AttackDamageType;
};

var array<AttackInfo> AttackInformation;

var int AttackIndex;
var string SpawnMessage;
var L2GPBossZombieSpawner BossSpawner;

const SHAKE_CAMERA_BIG = 350;
const SHAKE_CAMERA_MED = 250;
const MIN_MAG_FOR_SHAKE = 20;
const MAX_SHAKE_DIST = 2000.0;
const ADJUST_RELATIVE_HEAD_Z = -0.8;

function SwapToBurnVictim();
simulated function SetupAppearance();
simulated function SetupBoltons();
simulated function SetupDialog();
simulated function SetupCollisionInfo();

simulated function PostBeginPlay()
{
    local int PlayerCount;
    local PersonPawn temp;

    Super.PostBeginPlay();

    foreach DynamicActors(class'PersonPawn', temp)
        if (temp != None)
            PlayerCount++;

    if (PlayerCount == 0)
        return;

    log(string(self) $ "'s original health: " $ string(Health));

    HealthMax *= PlayerCount;
    Health *= PlayerCount;

    log(string(self) $ "'s new health: " $ string(Health));
}

simulated function vector GetOffset(vector Offset)
{
    local vector Xaxis, Yaxis, Zaxis;

    GetAxes(Rotation, Xaxis, Yaxis, Zaxis);

    return Location + Xaxis*Offset.X + Yaxis*Offset.Y + Zaxis*Offset.Z;
}

simulated function SetupHead()
{
    local rotator RelativeHeadRotation;
    local vector RelativeHeadLocation;

    if (HeadClass != None && HeadMesh != None)
    {
        myHead = Spawn(HeadClass,,, GetBoneCoords(BONE_HEAD).Origin);

        if (myHead != None)
        {
            myHead.Setup(HeadMesh, HeadSkin, HeadScale, AmbientGlow);
            AttachToBone(myHead, BONE_HEAD);

            RelativeHeadRotation.Pitch = 0;
			RelativeHeadRotation.Yaw = -16384;
			RelativeHeadRotation.Roll = 16384;
			myHead.SetRelativeRotation(RelativeHeadRotation);

			RelativeHeadLocation.x = ADJUST_RELATIVE_HEAD_Z;
			RelativeHeadLocation.y = 0;
			RelativeHeadLocation.z = 0;
			myHead.SetRelativeLocation(RelativeHeadLocation);
        }
    }
}

simulated function TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
    if (Health <= 0 && BossSpawner != None)
        BossSpawner.SetBossTimer();
		ZombieKilled(EventInstigator.Controller, DamageType, HitLocation);
}

simulated function ZombieKilled(Controller KillerController, class<DamageType> DamageType, vector HitLocation)
{
    SetCollision(false, false, false);

    if (Controller != None)
        Controller.Destroy();

    KillerController.PlayerReplicationInfo.Score += 25;
    KillerController.PlayerReplicationInfo.Kills++ ;

    PlayDying(DamageType, HitLocation);

    Controller.GotoState('Dying');
    GotoState('Dying');
}

function ShakeCameraDistanceBased(float Mag)
{
	local controller con;
	local float usemag, usedist;

	for(con = Level.ControllerList; con != None; con=con.NextController)
	{
		// Find who did it first, then shake them
		if(con.bIsPlayer && Con.Pawn!=None
			&& Con.Pawn.Physics != PHYS_FALLING)
		{
			usedist = VSize(con.Pawn.Location - Location);
			if(usedist > MAX_SHAKE_DIST)
				usedist = MAX_SHAKE_DIST;
			usemag = ((MAX_SHAKE_DIST - usedist)/MAX_SHAKE_DIST)*Mag;
			//log("use mag "$usemag);
			if(usemag < MIN_MAG_FOR_SHAKE)
				return;

			con.ShakeView((usemag * 0.2 + 1.0)*vect(1.0,1.0,3.0),
               vect(1000,1000,1000),
               1.0 + usemag*0.02,
               (usemag * 0.3 + 1.0)*vect(1.0,1.0,2.0),
               vect(800,800,800),
               1.0 + usemag*0.02);
		}
	}
}

simulated event SetAnimAction(name NewAction)
{
    AnimAction = NewAction;

    PlayAnim(AnimAction);
}

simulated function EWeaponHoldStyle GetWeaponHoldStyle()
{
    return WEAPONHOLDSTYLE_None;
}

simulated function EWeaponHoldStyle GetWeaponSwitchStyle()
{
    return WEAPONHOLDSTYLE_None;
}

simulated function EWeaponHoldStyle GetWeaponFiringStyle()
{
    return WEAPONHOLDSTYLE_None;
}

defaultproperties
{
     bRandomizeHeadScale=False
}
