////////////////////////////////////////////
// Res-Studios.net
///////////////////////////////////////////
class L2GPZombie extends MpPawn;

struct IdleAnimInfo
{
    var() name IdleAnim;
    var() float IdleAnimRate;
};

struct WalkAnimInfo
{
    var() name WalkAnim;
    var() float WalkAnimRate;
};

struct RunAnimInfo
{
    var() name RunAnim;
    var() float RunAnimRate;
};

struct AttackInfo
{
    var() name AttackAnim;
    var() float AttackAnimRate;
    var() float AttackDelay;
    var() float AttackDuration;
    var() float AttackDamage;
    var() class<DamageType> AttackDamageType;
};

var() bool bRuns;

var() array<IdleAnimInfo> IdleAnims;
var() array<WalkAnimInfo> WalkAnims;
var() array<RunAnimInfo> RunAnims;
var() array<AttackInfo> AttackInformation;

var() float AttackRange;
var() float WalkSpeed;
var() float RunSpeed;

var() sound GibSound;

var() array<sound> AngerSounds;
var() array<sound> CurseSounds;
var() array<sound> MoanSounds;
var() array<sound> PainSounds;

var() array<texture> HeadSkins;

var int IdleAnimIndex;
var int WalkAnimIndex;
var int RunAnimIndex;
var int AttackIndex;

const ADJUST_RELATIVE_HEAD_Z = -0.8;

simulated function PostBeginPlay()
{
    Super.PostBeginPlay();

    if (IdleAnims.length > 0)
        IdleAnimIndex = Rand(IdleAnims.length);

    if (WalkAnims.length > 0)
        WalkAnimIndex = Rand(WalkAnims.length);

    if (RunAnims.length > 0)
        RunAnimIndex = Rand(RunAnims.length);

    if (bRuns)
        GroundSpeed = RunSpeed;
    else
        GroundSpeed = WalkSpeed;

    ChangeAnimation();
    Timer();
}

simulated function SetupAppearance();
simulated function SetupBoltons();
simulated function SetupDialog();
simulated function SetupCollisionInfo();

simulated function SetupHead()
{
    local int HeadIndex;
    local rotator RelativeHeadRotation;
    local vector RelativeHeadLocation;

    if (HeadClass != None && HeadMesh != None)
    {
        myHead = Spawn(HeadClass,,, GetBoneCoords(BONE_HEAD).Origin);

        if (myHead != None)
        {
            if (HeadSkins.length > 0)
            {
                HeadIndex = Rand(HeadSkins.length);
                HeadSkin = HeadSkins[HeadIndex];
            }

            // Used mostly for debugging
            //if (HeadSkin == None)
               // log("Defective Head Skin: "$string(HeadIndex));

            myHead.Setup(HeadMesh, HeadSkin, HeadScale, AmbientGlow);
            AttachToBone(myHead, BONE_HEAD);

            RelativeHeadRotation.Pitch = 0;
			RelativeHeadRotation.Yaw = -16384;
			RelativeHeadRotation.Roll = 16384;
			myHead.SetRelativeRotation(RelativeHeadRotation);

			RelativeHeadLocation.x = ADJUST_RELATIVE_HEAD_Z;
			RelativeHeadLocation.y = 0;
			RelativeHeadLocation.z = 0;
			myHead.SetRelativeLocation(RelativeHeadLocation);
        }
    }
}

///attempting to fix error

simulated function DestroyHead()
{
	if(myHead != None)
	myHead.Destroy();
}

simulated function UpdateRun(bool bShouldRun)
{
    bRuns = bShouldRun;

    if (bRuns)
        GroundSpeed = RunSpeed;
    else
        GroundSpeed = WalkSpeed;
}

simulated function Timer()
{
    local int CatagoryIndex;
    local int SoundIndex;

    CatagoryIndex = Rand(3);

    switch (CatagoryIndex)
    {
        case 0:    SoundIndex = Rand(AngerSounds.length);
                   PlaySound(AngerSounds[SoundIndex]);
                   SetTimer(GetSoundDuration(AngerSounds[SoundIndex])*5, false);
                   break;

        case 1:    SoundIndex = Rand(CurseSounds.length);
                   PlaySound(CurseSounds[SoundIndex]);
                   SetTimer(GetSoundDuration(CurseSounds[SoundIndex])*5, false);
                   break;

        case 2:    SoundIndex = Rand(MoanSounds.length);
                   PlaySound(MoanSounds[SoundIndex]);
                   SetTimer(GetSoundDuration(MoanSounds[SoundIndex])*5, false);
                   break;
    }
}

simulated function bool IsInAttackRange(Actor Other)
{
    if (VSize(Other.Location - Location) <= AttackRange)
        return true;
    else
        return false;
}

simulated function SetupAnims()
{
	LinkMeshAndAnims(Mesh, true);
    LinkAnims();
	//log("Initialized Animations");
}

simulated function LinkMeshAndAnims(Mesh NewMesh, optional bool bKeepAnimState)
{
    Super.LinkMeshAndAnims(NewMesh, bKeepAnimState);
    //log("Linked Skeletal Mesh");
}

simulated function PlayMoving()
{
    if (IsInState('Attacking'))
        SetAnimAttacking();
    else if (bRuns)
        SetAnimRunning();
    else
        SetAnimWalking();
}

simulated function SetAnimWalking()
{
    local int i;

    BaseMovementRate = GroundSpeed;

    if (WalkAnims.length > 0 && HasAnim(WalkAnims[WalkAnimIndex].WalkAnim))
        for (i=0;i<4;i++)
            MovementAnims[i] = WalkAnims[WalkAnimIndex].WalkAnim;

    //TurnLeftAnim = WalkAnims[WalkAnimIndex].WalkAnim;
    //TurnRightAnim = WalkAnims[WalkAnimIndex].WalkAnim;

    //log("Set Anim Walking, BaseMovementRate: "$BaseMovementRate);
}

simulated function SetAnimRunning()
{
    local int i;

    //log("Set Anim Running");
    BaseMovementRate = GroundSpeed;

    if (RunAnims.length > 0 && HasAnim(RunAnims[RunAnimIndex].RunAnim))
        for (i=0;i<4;i++)
            MovementAnims[i] = RunAnims[RunAnimIndex].RunAnim;

    //TurnLeftAnim = RunAnims[RunAnimIndex].RunAnim;
    //TurnRightAnim = RunAnims[RunAnimIndex].RunAnim;

    //log("Set Anim Running, BaseMovementRate: "$BaseMovementRate);
}

simulated function SetAnimAttacking()
{
    local int i;

    AttackIndex = Rand(AttackInformation.length);
    BaseMovementRate = GroundSpeed;

    if (AttackInformation.length > 0 && HasAnim(AttackInformation[AttackIndex].AttackAnim))
        for (i=0;i<4;i++)
            MovementAnims[i] = AttackInformation[AttackIndex].AttackAnim;

    //TurnLeftAnim = AttackInformation[AttackIndex].AttackAnim;
    //TurnRightAnim = AttackInformation[AttackIndex].AttackAnim;
}

simulated event SetAnimAction(name NewAction)
{
    AnimAction = NewAction;

    PlayAnim(AnimAction);
}

/*
simulated event SetAnimAction(name NewAction)
{
    AnimAction = NewAction;

    if (AnimAction == 'sg_pull_pin')
    {
        WeaponBlendTime = FIRING_BLEND_TIME;
        AnimBlendParams(WEAPONCHANNEL, 1.0, 0,0, BONE_BLENDFIRING);
        PlayAnim(NewAction, 1.0, 0.1, WEAPONCHANNEL);
    }
    else if (AnimAction == 's_suicide2')
        PlayAnim(AnimAction, 1.5, 0.15);
    else if (AnimAction == GetAnimKick())
        PlayAnim(AnimAction, AttackInformation[AttackIndex].AttackAnimRate);
    else if (AnimAction == BURN_ACTION_MP)
        SwapToBurnVictim();
    else if (AnimAction == FOLLOW_ME)
        DoFollowMe();
    else if (AnimAction == STAY_HERE)
        DoStayHere();
}
*/

simulated function name GetAnimIdle()
{
    ////log("Returned Idle Animations");
    return IdleAnims[IdleAnimIndex].IdleAnim;
}

simulated function name GetAnimStand()
{
    //log("Returned Idle Animations");
    return IdleAnims[IdleAnimIndex].IdleAnim;
}

simulated function name GetAnimKick()
{
    //log("Returned Kick Anim");
    AttackIndex = Rand(AttackInformation.length);

    return AttackInformation[AttackIndex].AttackAnim;
}

simulated function EWeaponHoldStyle GetWeaponHoldStyle()
{
    //log("Returned WEAPONHOLDSTYLE_None");
    return WEAPONHOLDSTYLE_None;
}

simulated function EWeaponHoldStyle GetWeaponSwitchStyle()
{
    //log("Returned WEAPONHOLDSTYLE_None");
    return WEAPONHOLDSTYLE_None;
}

simulated function EWeaponHoldStyle GetWeaponFiringStyle()
{
    //log("Returned WEAPONHOLDSTYLE_None");
    return WEAPONHOLDSTYLE_None;
}

event Bump(Actor Other)
{
    if (Other != None && Other.IsA('Pawn') && !Other.IsA('L2GPZombie') && !Other.IsA('AnimalPawn') && !Other.IsA('L2GPBossZombie') && Pawn(other).Health > 0)
    {
        Controller.Enemy = Pawn(Other);
        GotoState('Attacking');
    }
}

simulated function TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
    if (PainSounds.length > 0)
        PlaySound(PainSounds[Rand(PainSounds.length)]);

    if (Controller.Enemy == None)
    {
        Controller.Enemy = EventInstigator;

        if (IsInAttackRange(Controller.Enemy))
        {
            Controller.GotoState('Attacking');
            GotoState('Attacking');
        }
        else
        {
            Controller.GotoState('Chasing');
            GotoState('Chasing');
        }
    }
    //ChunkUp(0);
    Health -= Damage;

    Spawn(class'BloodImpactMP',,, HitLocation, rotator(Momentum));

    if (Health <= 0)
		ZombieKilled(EventInstigator.Controller, DamageType, HitLocation);
}

simulated function ChunkUp(int Damage)
{
	local CatExplosion exp;
   // local DonutPickup Steak;
	bChunkedUp = true;

	if (class'P2Player'.static.BloodMode())
	{
	    //Spawn(class'DonutPickup',,,Location);
		exp = Spawn(class'PawnExplosion',,,Location);
		//foreach DynamicActors(class'DonutPickup', Steak)
		//	if (Steak != None)
		  //  {
		    //    Steak.SetPhysics(PHYS_Falling);
		    //}
			
			
		if(exp != None)
		{
			exp.ReduceMagBasedOnProx(Location, 1.0);//hitmag);
			exp.PlaySound(GibSound);
		}
	}
	
	//spawn(class'RocketSmokePuff',,,Location);	// gotta give the lame no-blood mode something!

	Super.ChunkUp(Damage);

	//if (myHead != None)
	//    myHead.Destroy();

    Destroy();
}

simulated function ZombieKilled(Controller KillerController, class<DamageType> DamageType, vector HitLocation)
{
    ChunkUp(0);
    SetCollision(false, false, false);

    if (Controller != None)
        Controller.Destroy();

    KillerController.PlayerReplicationInfo.Score += 1;
    KillerController.PlayerReplicationInfo.Kills++ ;

    PlayDying(DamageType, HitLocation);

    Controller.GotoState('Dead');
    GotoState('Dead');
}

event KImpact(actor other, vector pos, vector impactVel, vector impactNorm)
{
	local Actor HitActor;
	local vector checkpoint, HitLocation, HitNormal;

	//log(self$" hit this hard "$impactVel$" mag "$VSize(impactVel));
	// Make hit noises
	if(Level.TimeSeconds > (LastBodyHitTime + TimeBetweenPainSounds))
	{
		PlaySound(BodyHitSounds[Rand(BodyHitSounds.Length)], SLOT_Pain, 1.0, , 100, GetRandPitch());
		LastBodyHitTime = Level.TimeSeconds;

		// From the big impact, put a blood splat
		checkpoint = Location - DIST_TO_WALL_FOR_BLOODSPLAT*Normal(impactNorm);

		HitActor = Trace(HitLocation, HitNormal, checkpoint, Location, true);

		if ( HitActor != None
			&& HitActor.bStatic
			&& class'P2Player'.static.BloodMode())
		{
			spawn(class'BloodExplosionSplatMaker',self,,HitLocation,rotator(HitNormal));
		}
	}
}

simulated state Chasing
{
    simulated function BeginState()
    {
        //log("Zombie: Entered Chase State");

        ChangeAnimation();
    }

    simulated function Tick(float DeltaTime)
    {
        if (Controller.Enemy != None)
        {
            if (IsInAttackRange(Controller.Enemy))
            {
                Controller.GotoState('Attacking');
                GotoState('Attacking');
            }
        }
        else
        {
            Controller.Enemy = None;
            Controller.GotoState('Idle');
            GotoState('');
        }
    }
}

simulated state Attacking
{
    simulated function BeginState()
    {
        local int i;

        //log("Zombie: Entered Attack State");

        for (i=0;i<4;i++)
            MovementAnims[i] = '';

        /*
        if (AttackInformation.length > 0)
        {
            StopAcc();
            PerformKick();
            PlaySound(AngerSounds[Rand(AngerSounds.length)]);
            SetTimer(AttackInformation[AttackIndex].AttackDelay, false);
        }
        */
    }

    simulated function DoDamage()
    {
        if (IsInAttackRange(Controller.Enemy))
            Controller.Enemy.TakeDamage(AttackInformation[AttackIndex].AttackDamage,self,Controller.Enemy.Location,vect(0,0,0),AttackInformation[AttackIndex].AttackDamageType);
    }

    /*
    simulated function AnimEnd(int Channel)
    {
        Super.AnimEnd(Channel);

        if (Channel == WEAPONCHANNEL)
        {
        if (Controller.Enemy.Health <= 0)
        {
            Controller.Enemy = None;
            Controller.GotoState('Idle');
            GotoState('');
        }
        else if (!IsInAttackRange(Controller.Enemy))
        {
            Controller.GotoState('Chasing');
            GotoState('Chasing');
        }
        else if (AttackInformation.length > 0)
        {
            StopAcc();
            PerformKick();
            PlaySound(AngerSounds[Rand(AngerSounds.length)]);
            SetTimer(AttackInformation[AttackIndex].AttackDelay, false);
        }
        }
    }
    */
Begin:
    Goto 'AttackLoop';

AttackLoop:
    StopAnimating();
    StopAcc();
    PerformKick();
    PlaySound(AngerSounds[Rand(AngerSounds.length)]);
    Sleep(AttackInformation[AttackIndex].AttackDelay);
    DoDamage();
    Sleep(AttackInformation[AttackIndex].AttackDuration - AttackInformation[AttackIndex].AttackDelay);

    if (IsInAttackRange(Controller.Enemy))
        Goto 'AttackLoop';
    else
    {
        Controller.GotoState('Chasing');
        GotoState('Chasing');
    }
}

simulated state Dead
{
Begin:
    Sleep(1.0);

    if (myHead != None)
	    myHead.Destroy();

    Destroy();
}

defaultproperties
{
     bRuns=True
	 bChameleon=False
     AttackRange=96.000000
     WalkSpeed=112.000000
     RunSpeed=450.000000
     GibSound=Sound'WeaponSounds.flesh_explode'
     bRandomizeHeadScale=False
     HealthMax=200.000000
     Health=200
     ControllerClass=Class'Left2GoPostal.L2GPZombieController'
     MovementAnims(0)="None"
     MovementAnims(1)="None"
     MovementAnims(2)="None"
     MovementAnims(3)="None"
     TurnLeftAnim="None"
     TurnRightAnim="None"
     Physics=PHYS_Walking
     bEdShouldSnap=True
	 ChameleonMeshPkgs(0)="None"
}
