////////////////////////////////////////////
// Res-Studios.net
///////////////////////////////////////////
class L2GPZombieSpawner extends PathNode
    placeable;

var() float SpawnInterval;
var() float SpawnCheckRadius;
var() float ChanceOfRunning;
var() int MaxZombiePopulation;
var() name InitialNodeTag;

var() array< class<L2GPZombie> > ZombieClasses;

var array<L2GPPathNode> InitialPathNodes;

var L2GPAIDirector AIDirector;

simulated function PostBeginPlay()
{
    Super.PostBeginPlay();

    Initialize(None);

    GotoState('FreeSpawn');
}

simulated function Initialize(L2GPAIDirector Director)
{
    SetupInitialNodes();

    if (Director != None)
        AIDirector = Director;
}

simulated function SetupInitialNodes()
{
    local PathNode tempNode;
    local L2GPPathNode L2GPNode;

    if (InitialNodeTag != '' || InitialNodeTag != 'None')
    {
        foreach AllActors(class'L2GPPathNode', L2GPNode, InitialNodeTag)
            if (L2GPNode != None)
            {
                InitialPathNodes.Insert(InitialPathNodes.length,1);
                InitialPathNodes[InitialPathNodes.length-1] = L2GPNode;
            }
    }
}

simulated function int GetZombiePopulation()
{
    local int ZombieCount;
    local L2GPZombie Zombie;

    foreach DynamicActors(class'L2GPZombie', Zombie)
        if (Zombie.Health > 0)
            ZombieCount++;

    return ZombieCount;
}

simulated function PersonPawn GetClosestPlayer()
{
    return None;
}

simulated function bool CanBeSeen()
{
    local PersonPawn Player;

    foreach DynamicActors(class'PersonPawn', Player)
        if (FastTrace(Player.Location))
            return true;

    return false;
}

simulated function bool ClearForSpawn()
{
    local L2GPZombie Zombie;

    foreach VisibleCollidingActors(class'L2GPZombie', Zombie, SpawnCheckRadius)
        if (Zombie != None)
            return false;

    return true;
}

simulated function SpawnZombie()
{
    local L2GPZombie Zombie;
    local Controller ZombieController;

    if (CanBeSeen())
        return;

    if (GetZombiePopulation() >= MaxZombiePopulation)
        return;

    Zombie = Spawn(ZombieClasses[Rand(ZombieClasses.length)]);

    if (Zombie != None)
    {
        ZombieController = Spawn(Zombie.default.ControllerClass);
        Zombie.SetPhysics(PHYS_Falling);

        if (FRand() < ChanceOfRunning)
            Zombie.UpdateRun(true);
        else
            Zombie.UpdateRun(false);
    }

    if (ZombieController != None)
    {
        if (InitialPathNodes.length > 0 && IsInState('FreeSpawn'))
            L2GPZombieController(ZombieController).InitialPathNode = InitialPathNodes[Rand(InitialPathNodes.length)];
        else if (AIDirector != None && IsInState('HoardSpawn'))
            L2GPZombieController(ZombieController).SetEnemy(AIDirector.GetRandomSurvivor());

        ZombieController.Possess(Zombie);
    }
}

simulated function Timer()
{
    SpawnZombie();
}

simulated state FreeSpawn
{
Begin:
    SpawnZombie();

    Sleep(SpawnInterval);

    Goto 'Begin';
}

simulated state HoardSpawn
{
    simulated function BeginState()
    {
        SetTimer(SpawnInterval, true);
    }

    simulated function Timer()
    {
        SpawnZombie();
    }
}

simulated state NormalSpawn
{
Begin:
    SpawnZombie();
}

simulated state Rest
{
Begin:
}

defaultproperties
{
     SpawnInterval=0.500000
     SpawnCheckRadius=96.000000
     ChanceOfRunning=1.000000
     MaxZombiePopulation=10
     ZombieClasses(0)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
     bStatic=False
     bNoDelete=False
     Texture=Texture'Engine.S_Actor'
     bEdShouldSnap=True
}
