////////////////////////////////////////////
// Res-Studios.net
///////////////////////////////////////////
class L2GPZombieSpawner2 extends Actor
    placeable
	config(XLXtras1409);

var() config bool bSurvivalMode;
var() config float SpawnInterval;
var() config float SpawnCheckRadius;
var() config float ChanceOfRunning;
var() config int MaxZombiePopulation;
var() config name InitialNodeTag;
var() config array< class<L2GPZombie> > ZombieClasses;
var() config array<L2GPPathNode> InitialPathNodes;


const CHECK_FOR_GROUND_DIST	=	2048;


simulated function PostBeginPlay()
{
    if (ZombieClasses.length > 0)
    {
        SetTimer(SpawnInterval, true);
        SetupInitialNodes();
    }
}

simulated function SetupInitialNodes()
{
    local PathNode tempNode;
    local L2GPPathNode L2GPNode;

    if (InitialNodeTag != '' || InitialNodeTag != 'None')
    {
        foreach AllActors(class'L2GPPathNode', L2GPNode, InitialNodeTag)
            if (L2GPNode != None)
            {
                InitialPathNodes.Insert(InitialPathNodes.length,1);
                InitialPathNodes[InitialPathNodes.length-1] = L2GPNode;
            }
    }
}

simulated function int GetZombiePopulation()
{
    local int ZombieCount;
    local L2GPZombie Zombie;

    foreach DynamicActors(class'L2GPZombie', Zombie)
        if (Zombie.Health > 0)
            ZombieCount++;

    return ZombieCount;
}

simulated function PersonPawn GetClosestPlayer()
{
    return None;
}

simulated function bool CanBeSeen()
{
    local PersonPawn Player;

    foreach DynamicActors(class'PersonPawn', Player)
        if (FastTrace(Player.Location))
            return true;

    return false;
}

simulated function bool ClearForSpawn()
{
    local L2GPZombie Zombie;

    foreach VisibleCollidingActors(class'L2GPZombie', Zombie, SpawnCheckRadius)
        if (Zombie != None)
            return false;

    return true;
}

simulated function Timer()
{
    local L2GPZombie Zombie;
    local Controller ZombieController;

    if (CanBeSeen())
        return;

    if (GetZombiePopulation() >= MaxZombiePopulation)
        return;

    Zombie = Spawn(ZombieClasses[Rand(ZombieClasses.length)]);

    if (Zombie != None)
    {
        ZombieController = Spawn(Zombie.default.ControllerClass);
        Zombie.SetPhysics(PHYS_Falling);

        if (FRand() < ChanceOfRunning)
            Zombie.UpdateRun(true);
        else
            Zombie.UpdateRun(false);
    }

    if (ZombieController != None)
    {
        if (InitialPathNodes.length > 0)
            L2GPZombieController(ZombieController).InitialPathNode = InitialPathNodes[Rand(InitialPathNodes.length)];

        ZombieController.Possess(Zombie);
    }
}


function FindGround()
{
	local Actor HitActor;
	local vector HitLocation, HitNormal, checkpos;
	local Rotator userot;

	checkpos = Location;
	checkpos.z -= CHECK_FOR_GROUND_DIST;

	HitActor = Trace(HitLocation, HitNormal, checkpos, Location, false);
	if(HitActor != None)
	{
		HitLocation+=CollisionHeight*HitNormal;
		SetLocation(HitLocation);
	}
}
defaultproperties
{
     SpawnInterval=0.100000
SpawnCheckRadius=120.000000
ChanceOfRunning=0.500000
MaxZombiePopulation=50
ZombieClasses(0)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
ZombieClasses(1)=Class'Left2GoPostal.L2GPZombieAW_MaleBystander'
ZombieClasses(2)=Class'Left2GoPostal.L2GPZombieAW_MaleSuit'
ZombieClasses(3)=Class'Left2GoPostal.L2GPZombieAW_Overcoat'
ZombieClasses(4)=Class'Left2GoPostal.L2GPZombieAW_Plastinate'
ZombieClasses(5)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(6)=Class'Left2GoPostal.L2GPZombieAW7_MaleBystander'
ZombieClasses(7)=Class'Left2GoPostal.L2GPZombieAW7_MaleSuit'
ZombieClasses(8)=Class'Left2GoPostal.L2GPZombieAW7_Overcoat'
ZombieClasses(9)=Class'Left2GoPostal.L2GPZombieAW7_Plastinate'
ZombieClasses(10)=Class'Left2GoPostal.L2GPZombieAW7_Krotchy'
ZombieClasses(11)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(12)=Class'Left2GoPostal.L2GPZombieAW7_Taliban'
ZombieClasses(13)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
ZombieClasses(14)=Class'Left2GoPostal.L2GPZombieAW_MaleBystander'
ZombieClasses(15)=Class'Left2GoPostal.L2GPZombieAW_MaleSuit'
ZombieClasses(16)=Class'Left2GoPostal.L2GPZombieAW_Overcoat'
ZombieClasses(17)=Class'Left2GoPostal.L2GPZombieAW_Plastinate'
ZombieClasses(18)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(19)=Class'Left2GoPostal.L2GPZombieAW7_MaleBystander'
ZombieClasses(20)=Class'Left2GoPostal.L2GPZombieAW7_MaleSuit'
ZombieClasses(21)=Class'Left2GoPostal.L2GPZombieAW7_Overcoat'
ZombieClasses(22)=Class'Left2GoPostal.L2GPZombieAW7_Plastinate'
ZombieClasses(23)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(24)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
ZombieClasses(25)=Class'Left2GoPostal.L2GPZombieAW_MaleBystander'
ZombieClasses(26)=Class'Left2GoPostal.L2GPZombieAW_MaleSuit'
ZombieClasses(27)=Class'Left2GoPostal.L2GPZombieAW_Overcoat'
ZombieClasses(28)=Class'Left2GoPostal.L2GPZombieAW_Plastinate'
ZombieClasses(29)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(30)=Class'Left2GoPostal.L2GPZombieAW7_MaleBystander'
ZombieClasses(31)=Class'Left2GoPostal.L2GPZombieAW7_MaleSuit'
ZombieClasses(32)=Class'Left2GoPostal.L2GPZombieAW7_Overcoat'
ZombieClasses(33)=Class'Left2GoPostal.L2GPZombieAW7_Plastinate'
ZombieClasses(34)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
ZombieClasses(35)=Class'Left2GoPostal.L2GPZombieAW_MaleBystander'
ZombieClasses(36)=Class'Left2GoPostal.L2GPZombieAW_MaleSuit'
ZombieClasses(37)=Class'Left2GoPostal.L2GPZombieAW_Overcoat'
ZombieClasses(38)=Class'Left2GoPostal.L2GPZombieAW_Plastinate'
ZombieClasses(39)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(40)=Class'Left2GoPostal.L2GPZombieAW7_MaleBystander'
ZombieClasses(41)=Class'Left2GoPostal.L2GPZombieAW7_MaleSuit'
ZombieClasses(42)=Class'Left2GoPostal.L2GPZombieAW7_Overcoat'
ZombieClasses(43)=Class'Left2GoPostal.L2GPZombieAW7_Plastinate'
ZombieClasses(44)=Class'Left2GoPostal.L2GPZombieAW_FemaleBystander'
ZombieClasses(45)=Class'Left2GoPostal.L2GPZombieAW_MaleBystander'
ZombieClasses(46)=Class'Left2GoPostal.L2GPZombieAW_MaleSuit'
ZombieClasses(47)=Class'Left2GoPostal.L2GPZombieAW_Overcoat'
ZombieClasses(48)=Class'Left2GoPostal.L2GPZombieAW_Plastinate'
ZombieClasses(49)=Class'Left2GoPostal.L2GPZombieAW7_FemaleBystander'
ZombieClasses(50)=Class'Left2GoPostal.L2GPZombieAW7_MaleBystander'
     bHidden=True
     bEdShouldSnap=True
}
